--
-- @author:  	gEtOvEr@LSSA
-- @edit: werik, igor29381
-- @version 	v2.4	fs17
-- @date  	27.10.16

Lock = {};

function Lock.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function Lock:load(savegame)

	local rotationPartNoderichag = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.richag#index"));
    if rotationPartNoderichag ~= nil then
        self.rotationPartrichag = {};
        self.rotationPartrichag.node = rotationPartNoderichag;

        local x, y, z = Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.richag#minRot"));
        self.rotationPartrichag.minRot = {};
        self.rotationPartrichag.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotationPartrichag.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotationPartrichag.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.richag#maxRot"));
        self.rotationPartrichag.maxRot = {};
        self.rotationPartrichag.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotationPartrichag.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotationPartrichag.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotationPartrichag.rotTime = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.richag#rotTime"), 2)*1000;
        self.rotationPartrichag.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.richag#touchRotLimit"), 10));
    end;
	self.JointLock = false;
end;

function Lock:update(dt)
	local isActive = false;

	local jointDesc = self.componentJoints[1];
	local px, py, pz = getWorldTranslation(jointDesc.jointNode);
	if not self:getIsActiveForInput() then
		if g_currentMission.player ~= nil then
			local pvx, pvy, pvz = getWorldTranslation(g_currentMission.player.rootNode);
			local farmerDistance = Utils.vector3Length(pvx-px, pvy-py, pvz-pz);
			if farmerDistance < 3 then
				isActive = true;
				if not self.JointLock then
					g_currentMission:addHelpButtonText(g_i18n:getText("Lock"), InputBinding.TOGGLE_TENSION_BELTS);
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("UnLock"), InputBinding.TOGGLE_TENSION_BELTS);
				end;
			end;
		end;
	end;

	if isActive then
		if InputBinding.hasEvent(InputBinding.TOGGLE_TENSION_BELTS) then
			local x,y,z = getRotation(self.components[1].node);
			local x1,y1,z1 = getRotation(self.components[2].node);
			if math.abs(y - y1) < 0.01 then
				self.JointLock = not self.JointLock;
				self.rotationMaxrichag = self.JointLock;
				if g_server then
					if self.JointLock then
						setJointRotationLimit(self.componentJoints[1].jointIndex, 1, true, 0, 0);
					else
						setJointRotationLimit(self.componentJoints[1].jointIndex, 1, true, math.rad(-120), math.rad(120));
					end;
					g_server:broadcastEvent(LockEvent:new(self.JointLock, self));
				else
					g_client:getServerConnection():sendEvent(LockEvent:new(self.JointLock, self));
				end;
			else
				g_currentMission:showBlinkingWarning(g_i18n:getText("NoLock"), 2000);
			end;
		end;
	end;

	if self.rotationPartrichag ~= nil then
		local x, y, z = getRotation(self.rotationPartrichag.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartrichag.maxRot, self.rotationPartrichag.minRot, 3, self.rotationPartrichag.rotTime, dt, not self.rotationMaxrichag);
		setRotation(self.rotationPartrichag.node, unpack(newRot));
	end;
end;

function Lock:updateTick()
end;

function Lock:draw()
end;

function Lock:delete()
end;

function Lock:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Lock:keyEvent(unicode, sym, modifier, isDown)
end;

LockEvent = {};
LockEvent_mt = Class(LockEvent, Event);
InitEventClass(LockEvent, "LockEvent");

function LockEvent:emptyNew()
	local self = Event:new(LockEvent_mt);
    return self;
end;

function LockEvent:new(JointLock, vehicle)
	local self = LockEvent:emptyNew();
	self.JointLock = JointLock;
	self.vehicle = vehicle;
	return self;
end;

function LockEvent:readStream(streamId, connection)
	local JointLock = streamReadBool(streamId);
	local id = streamReadInt32(streamId);
    local vehicle = networkGetObject(id);
	if vehicle then
		vehicle.JointLock = JointLock;
		vehicle.rotationMaxrichag = JointLock;
		if g_server then
			if JointLock then
				setJointRotationLimit(vehicle.componentJoints[1].jointIndex, 1, true, 0, 0);
			else
				setJointRotationLimit(vehicle.componentJoints[1].jointIndex, 1, true, math.rad(-120), math.rad(120));
			end;
			g_server:broadcastEvent(LockEvent:new(JointLock, vehicle));
		end;
	end;
end;

function LockEvent:writeStream(streamId, connection)
	streamWriteBool(streamId, self.JointLock);
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
end;
